@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001, 2002, 2003, 2004 Philip Blundell <philb@gnu.org>
- * Hildon adaption 2005 by Florian Boor <florian.boor@kernelconcepts.de>
+ * Copyright (C) 2005, 2006, 2007, 2008 by Florian Boor <florian.boor@kernelconcepts.de>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -9,7 +9,7 @@
  */
 
 /*
- * $Id: main-hildon.c,v 1.9 2006/06/10 15:57:29 florian Exp $
+ * $Id: main-hildon.c 9413 2008-05-20 20:03:58Z fboor $
  *
  */
 
@@ -34,17 +34,34 @@
 #include <gpe/picturebutton.h>
 #include <gpe/spacing.h>
 
+#include <gpe/contacts-db.h>
+
+#include "config.h"
+
 /* Hildon includes */
-#include <hildon-widgets/hildon-app.h>
-#include <hildon-widgets/hildon-appview.h>
-#include <gpe/pim-categories-ui.h>
+
+#if HILDON_VER > 0
+#include <hildon/hildon-file-chooser-dialog.h>
+#include <hildon/hildon-caption.h>
+#include <hildon/hildon-date-editor.h>
+#include <hildon/hildon-program.h>
+#include <hildon/hildon-window.h>
+#else
 #include <hildon-fm/hildon-widgets/hildon-file-chooser-dialog.h>
+#include <hildon-widgets/hildon-caption.h>
+#include <hildon-widgets/hildon-date-editor.h>
+#include <hildon-widgets/hildon-input-mode-hint.h>
+#include <hildon-widgets/hildon-program.h>
+#include <hildon-widgets/hildon-window.h>
+#include <hildon-widgets/hildon-file-chooser-dialog.h>
+#endif /* HILDON_VER */
 
 #include <libosso.h>
 
+#include <gpe/pim-categories-ui.h>
+
 
 #include "support.h"
-#include <gpe/contacts-db.h>
 #include "structure.h"
 #include "proto.h"
 #include "main.h"
@@ -59,19 +76,20 @@
 #define SERVICE_VERSION VERSION
 
 
-GtkWidget *categories_smenu;
+static GtkWidget *categories_smenu;
 GtkWidget *mainw;
-GtkListStore *list_store, *filter_store;
-GtkWidget *list_view, *filter_view;
-GtkWidget *search_entry;
-GtkWidget *popup_menu;
-GtkWidget *bluetooth_menu_item;
-GtkWidget *irda_menu_item;
+static GtkListStore *list_store, *filter_store;
+static GtkWidget *list_view, *filter_view;
+static GtkWidget *search_entry;
+static GtkWidget *popup_menu;
+static GtkWidget *bluetooth_menu_item;
+static GtkWidget *irda_menu_item;
+static GtkWidget *fullscreen_control;
 guint menu_uid;
 gboolean mode_landscape;
 gboolean mode_large_screen;
-gboolean scroll_delay = FALSE;
-gboolean do_wrap = FALSE;
+static gboolean scroll_delay = FALSE;
+static gboolean do_wrap = FALSE;
 static osso_context_t *context = NULL;
 
 #define DEFAULT_STRUCTURE PREFIX "/share/gpe-contacts/default-layout.xml"
@@ -99,8 +117,17 @@ t_filter filter[]= {
   {"...", NULL},
 };
 
-int num_filter = sizeof(filter) / sizeof(t_filter);
-int current_filter = 0;
+gint num_filter = sizeof(filter) / sizeof(t_filter);
+gint current_filter = 0;
+
+static void
+toggle_fullscreen (GtkCheckMenuItem *menuitem, gpointer user_data)
+{
+  if (gtk_check_menu_item_get_active (menuitem))
+    gtk_window_fullscreen (GTK_WINDOW (mainw));
+  else
+    gtk_window_unfullscreen (GTK_WINDOW (mainw));
+}
 
 static void 
 populate_filter(void)
@@ -339,6 +366,7 @@ pop_singles (GtkWidget *vbox, GSList *list, struct contacts_person *p)
               if (table == NULL) 
                  table = gtk_table_new (length, 2, FALSE);
               w = gtk_label_new (NULL);
+              gtk_label_set_selectable(GTK_LABEL(w), TRUE);
               gtk_misc_set_alignment(GTK_MISC(w),0.0,0.5);
               gtk_label_set_text(GTK_LABEL(w), tv->value);
               gtk_label_set_line_wrap(GTK_LABEL(w), TRUE);
@@ -949,11 +977,20 @@ schedule_search (GObject *obj)
 void
 update_display (void)
 {
+  GtkTreeIter iter;
+  GtkTreePath *path = NULL;
   GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
+
+  gtk_tree_selection_get_selected (sel, NULL, &iter);
+  path = gtk_tree_model_get_path (gtk_tree_view_get_model(GTK_TREE_VIEW(list_view)), &iter);
   update_categories ();
-  if (sel)
-    selection_made(sel, G_OBJECT(mainw));
   do_search (G_OBJECT (search_entry), search_entry);
+  if (path)
+    {
+      gtk_tree_selection_select_path (sel, path);
+      gtk_tree_path_free (path);
+    }
+  selection_made(sel, G_OBJECT(mainw));
 }
 
 static gboolean
@@ -1095,6 +1132,10 @@ window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkTreeView *tree)
     {
       edit_contact(NULL, NULL);
     }
+  /* toggle fullscreen */
+  if (k->keyval == GDK_F6)
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (fullscreen_control), 
+             !gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (fullscreen_control)));
   
   /* automatic search */
   if (!k->state && k->string && isalpha(k->string[0]))
@@ -1252,11 +1293,11 @@ on_main_focus(GtkWindow *window,  GdkEventExpose *event,gpointer user_data)
 }
 
 static int 
-import_one_file(const gchar *filename)
+import_one_file(const gchar *filename, GError **error)
 {
   int result = 0;
 	
-  result = import_vcard(filename);
+  result = import_vcard(filename, error);
 
   return result;
 }
@@ -1272,30 +1313,35 @@ on_import_vcard (GtkWidget *widget, gpointer data)
   
   if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK)
     {
+      GError *error = NULL;
       gchar *errstr = NULL;
-      int ec = 0, i = 0;
-      gchar **files = 
+      gint ec = 0;
+      GSList *f;
+      GSList *files = 
         gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(filesel));
       gtk_widget_hide(filesel);
-      while (files && files[i])
+      f = files;
+      while (f && f->data)
         {
-          if (import_one_file(files[i]) < 0) 
+          if (import_one_file(f->data, &error) < 0) 
             {
               gchar *tmp;
               if (!errstr) 
                 errstr=g_strdup("");
               ec++;
-			  tmp = g_strdup_printf("%s\n%s", errstr, strrchr(files[i], '/')+1);
+              tmp = g_strdup_printf("%s\n%s: %s", errstr, 
+                                    strrchr(f->data,'/') + 1, error->message);
               if (errstr) 
                  g_free(errstr);
               errstr = tmp;
+              g_clear_error(&error);
             }
-          i++;  
+          f = f->next;
         }
       if (ec)
         feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(mainw)),
           GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
-          "%s %i %s\n%s",_("Import of"),ec,_("files failed:"),errstr);
+          "%s %i %s\n%s",_("Import of"), ec, _("files failed:"), errstr);
       else
         feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(mainw)),
           GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
@@ -1335,11 +1381,57 @@ edit_categories (GtkWidget *w)
   gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 }
 
+static void
+on_export_vcard (GtkWidget *widget, gpointer data)
+{
+  GtkWidget *filesel, *feedbackdlg;
+  gint i = 0, ec = 0;
+  GtkTreeIter iter;
+  gchar *filename = g_strconcat(g_get_home_dir(), "GPE.vcf", NULL);
+    
+  filesel = hildon_file_chooser_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(widget)), 
+                                                      GTK_FILE_CHOOSER_ACTION_SAVE);
+  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(filesel), TRUE);
+  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(filesel), filename);
+  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(filesel), FALSE);
+  
+  if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK)
+    {
+      guint uid;
+      gchar *filename = 
+        gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+      gtk_widget_hide(filesel);
+      
+      gtk_tree_model_get_iter_first (GTK_TREE_MODEL(list_store), &iter);
+      do 
+        {
+          gtk_tree_model_get (GTK_TREE_MODEL (list_store), &iter, 1, &uid, -1);
+          if (save_to_file (uid, filename, i ? TRUE : FALSE))
+             ec++;
+          i++;
+        }
+      while (gtk_tree_model_iter_next (GTK_TREE_MODEL(list_store), &iter));
+        
+      if (ec)
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
+          GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+          "%s %i %s %i",_("Export of"), i - ec, _("files ok, failed:"), ec);
+      else
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
+          GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+          _("Export successful"));
+      gtk_dialog_run(GTK_DIALOG(feedbackdlg));
+      gtk_widget_destroy(feedbackdlg);
+    }
+  g_free(filename);
+  gtk_widget_destroy(filesel);
+}
+
 /* create hildon application main menu */
 static void
-create_app_menu(HildonAppView *appview)
+create_app_menu(HildonWindow *window)
 {
-  GtkMenu *menu_main = hildon_appview_get_menu(appview);    
+  GtkMenu   *menu_main = GTK_MENU (gtk_menu_new());
   GtkWidget *menu_contacts = gtk_menu_new();
   GtkWidget *menu_categories = gtk_menu_new();
   GtkWidget *menu_view = gtk_menu_new();
@@ -1353,14 +1445,17 @@ create_app_menu(HildonAppView *appview)
   GtkWidget *item_add = gtk_menu_item_new_with_label(_("Add new"));
   GtkWidget *item_delete = gtk_menu_item_new_with_label(_("Delete"));
   GtkWidget *item_catedit = gtk_menu_item_new_with_label(_("Edit categories"));
+  GtkWidget *item_fullscreen = gtk_check_menu_item_new_with_label(_("Fullscreen"));
   GtkWidget *item_toolbar = gtk_check_menu_item_new_with_label(_("Show toolbar"));
   GtkWidget *item_tools = gtk_menu_item_new_with_label(_("Tools"));
   GtkWidget *item_import = gtk_menu_item_new_with_label(_("Import VCard"));
+  GtkWidget *item_export = gtk_menu_item_new_with_label(_("Export listed contacts..."));
 
   gtk_menu_append (GTK_MENU(menu_contacts), item_open);
   gtk_menu_append (GTK_MENU(menu_contacts), item_add);
   gtk_menu_append (GTK_MENU(menu_contacts), item_delete);
   gtk_menu_append (GTK_MENU(menu_categories), item_catedit);
+  gtk_menu_append (GTK_MENU(menu_view), item_fullscreen);
   gtk_menu_append (GTK_MENU(menu_view), item_toolbar);
   gtk_menu_append (menu_main, item_contacts);
   gtk_menu_append (menu_main, item_categories);
@@ -1368,22 +1463,28 @@ create_app_menu(HildonAppView *appview)
   gtk_menu_append (menu_main, item_tools);
   gtk_menu_append (menu_main, item_close);
   gtk_menu_append (menu_tools, item_import);
+  gtk_menu_append (menu_tools, item_export);
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_contacts), menu_contacts);
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_categories), menu_categories);
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_view), menu_view);
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item_tools), menu_tools);
   
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item_fullscreen), FALSE);
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item_toolbar), TRUE);
 
   g_signal_connect(G_OBJECT(item_add), "activate", G_CALLBACK(new_contact), NULL);
   g_signal_connect(G_OBJECT(item_open), "activate", G_CALLBACK(edit_contact), NULL);
   g_signal_connect(G_OBJECT(item_catedit), "activate", G_CALLBACK(edit_categories), NULL);
+  g_signal_connect(G_OBJECT(item_fullscreen), "activate", G_CALLBACK(toggle_fullscreen), NULL);
   g_signal_connect(G_OBJECT(item_toolbar), "activate", G_CALLBACK(toggle_toolbar), NULL);
   g_signal_connect(G_OBJECT(item_delete), "activate", G_CALLBACK(delete_contact), NULL);
   g_signal_connect(G_OBJECT(item_import), "activate", G_CALLBACK(on_import_vcard), NULL);
+  g_signal_connect(G_OBJECT(item_export), "activate", G_CALLBACK(on_export_vcard), NULL);
   g_signal_connect(G_OBJECT(item_close), "activate", G_CALLBACK(gtk_main_quit), NULL);
 
   gtk_widget_show_all (GTK_WIDGET(menu_main));  
+  fullscreen_control = item_fullscreen;
+  hildon_window_set_menu (HILDON_WINDOW (window), menu_main);
 }
 #endif
 
@@ -1406,9 +1507,9 @@ create_main (gboolean edit_structure)
   GtkWidget *lStatus = NULL;
   gint size_x, size_y;
 	
-  GtkWidget *app, *w;
-  GtkWidget *main_appview;
+  GtkWidget *w;
   GtkToolItem *item;
+  HildonProgram *app;
 
   /* screen layout detection */
   size_x = gdk_screen_width();
@@ -1419,15 +1520,14 @@ create_main (gboolean edit_structure)
   if (size_y < 320) size_y = 320;
 
   /* main application and appview */
-  app = hildon_app_new();
-  hildon_app_set_two_part_title(HILDON_APP(app), FALSE);
-  hildon_app_set_title (HILDON_APP(app), _("Contacts"));
-  main_appview = hildon_appview_new (_("Main"));
-  hildon_app_set_appview (HILDON_APP(app), HILDON_APPVIEW(main_appview));
-  main_window = main_appview;
-  create_app_menu (HILDON_APPVIEW(main_appview));
-  gtk_widget_show_all (app);
-  gtk_widget_show_all (main_appview);
+  app = HILDON_PROGRAM ( hildon_program_get_instance () );
+  g_set_application_name ( _("Contacts") );
+  main_window = hildon_window_new();
+  gtk_window_set_title (GTK_WINDOW (main_window), _("Main"));
+  hildon_program_add_window (app, HILDON_WINDOW (main_window));
+  gtk_widget_show_all (main_window);
+
+  create_app_menu (HILDON_WINDOW(main_window));
   
   vbox1 = gtk_vbox_new (FALSE, gpe_get_boxspacing());
   gtk_container_add (GTK_CONTAINER (main_window), vbox1);
@@ -1437,7 +1537,7 @@ create_main (gboolean edit_structure)
 			       GTK_ORIENTATION_HORIZONTAL);
 
   /* toolbar */
-  hildon_appview_set_toolbar(HILDON_APPVIEW(main_appview), GTK_TOOLBAR(toolbar));
+  hildon_window_add_toolbar(HILDON_WINDOW(main_window), GTK_TOOLBAR(toolbar));
   
   /* buttons left */
   w = gtk_image_new_from_file(ICON_PATH "/qgn_list_gene_unknown_file.png");
@@ -1466,6 +1566,10 @@ create_main (gboolean edit_structure)
   g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK (edit_categories), NULL);
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
 
+  item = gtk_tool_button_new_from_stock(GTK_STOCK_FIND);
+  g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(do_find), NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);    
+    
   item = gtk_separator_tool_item_new();
   gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(item), TRUE);
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
@@ -1516,7 +1620,7 @@ create_main (gboolean edit_structure)
   filter_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_set_size_request(filter_window, 60, -1);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (filter_window), 
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+                                  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
   filter_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (filter_store));
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (filter_view), FALSE);
   filter_sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (filter_view));
@@ -1636,7 +1740,8 @@ create_main (gboolean edit_structure)
     }
   tabDetail = gtk_table_new (1, 2, FALSE);
   gtk_table_set_col_spacings(GTK_TABLE(tabDetail), gpe_get_boxspacing());
-  gtk_container_set_border_width(GTK_CONTAINER(tabDetail), 0);
+  gtk_table_set_row_spacings(GTK_TABLE(tabDetail), gpe_get_boxspacing());
+  gtk_container_set_border_width(GTK_CONTAINER(tabDetail), gpe_get_border() / 2);
   if (mode_large_screen)
     {
       GtkWidget *vport;
@@ -1646,7 +1751,7 @@ create_main (gboolean edit_structure)
                                      GTK_POLICY_AUTOMATIC);
       vport = gtk_viewport_new(NULL, NULL);
       gtk_viewport_set_shadow_type(GTK_VIEWPORT(vport), GTK_SHADOW_NONE);
-      gtk_container_add(GTK_CONTAINER(vport), tabDetail);
+      gtk_container_add(GTK_CONTAINER (vport), tabDetail);
       gtk_container_add (GTK_CONTAINER (scrolled_window), vport);
       gtk_container_add (GTK_CONTAINER (pDetail), scrolled_window);
     }
@@ -1727,23 +1832,20 @@ main (int argc, char *argv[])
   if (ifile)
     {
       int ret;
-      GtkWidget *dialog = NULL;
+      GError *error = NULL;
       
-      ret =  import_one_file(ifile);
-      if (ret)
-       dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
-	                                    GTK_BUTTONS_OK,
-	                                    _("Could not import file %s."),
-	                                     ifile);
-	  else
-        dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
-	                                      GTK_BUTTONS_OK,
-	                                      _("File %s imported sucessfully."),
-	                                      ifile);
-      gtk_dialog_run(GTK_DIALOG(dialog));
-      gtk_widget_destroy(dialog);
-      
-      exit (EXIT_SUCCESS);
+      ret =  import_one_file(ifile, &error);
+      if (ret) 
+	{
+	  g_print(_("Could not import file %s: %s.\n"), ifile, error->message);
+	  g_error_free(error);
+	  exit (EXIT_FAILURE);
+	}
+      else
+	{
+	  g_print(_("File %s imported sucessfully.\n"), ifile);      
+	  exit (EXIT_SUCCESS);
+	}
     }
   
   export_init ();
@@ -1768,7 +1870,7 @@ main (int argc, char *argv[])
         p = contacts_new_person();
       edit_person (p, _("My Card"), TRUE);
       gtk_main();
-      if (save_to_file(1, MY_VCARD))
+      if (save_to_file(1, MY_VCARD, FALSE))
         gpe_perror_box(_("Saving vcard failed"));
       exit (EXIT_SUCCESS);
     }
