@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001, 2002, 2003, 2004 Philip Blundell <philb@gnu.org>
- *               2004, 2005, 2006 Florian Boor <florian@kernelconcepts.de>
+ *               2004, 2005, 2006, 2008 Florian Boor <florian@kernelconcepts.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -9,7 +9,7 @@
  */
 
 /*
- * $Id: main.c,v 1.130 2006/08/03 23:12:27 florian Exp $
+ * $Id: main.c 9504 2008-08-13 21:27:41Z philb $
  *
  */
 
@@ -281,6 +281,7 @@ pop_singles (GtkWidget *vbox, GSList *list, struct contacts_person *p)
               if (table == NULL) 
                  table = gtk_table_new (length, 2, FALSE);
               w = gtk_label_new (NULL);
+              gtk_label_set_selectable(GTK_LABEL(w), TRUE);
               gtk_misc_set_alignment(GTK_MISC(w),0.0,0.5);
               gtk_label_set_text(GTK_LABEL(w), tv->value);
               gtk_label_set_line_wrap(GTK_LABEL(w), 
@@ -415,7 +416,7 @@ build_children (GtkWidget *vbox, GSList *children, struct contacts_person *p)
           case ITEM_IMAGE:
           {
             GtkWidget *l = gtk_label_new (e->name);
-            GtkWidget *hbox, *image, *sw, *vp;
+            GtkWidget *hbox, *image;
             
             if (!(tv && tv->value))
               continue;
@@ -514,7 +515,7 @@ show_details (struct contacts_person *p)
         {
         
           if (c_old)
-            g_array_free (c_old, FALSE);
+            g_array_free (c_old, TRUE);
         
           g_object_set_data (G_OBJECT (cat_display), "catcolours", catcolours);
           gtk_widget_show (cat_display);
@@ -524,7 +525,7 @@ show_details (struct contacts_person *p)
         {
           g_object_set_data (G_OBJECT (cat_display), "catcolours", NULL);
           if (c_old)
-            g_array_free (c_old, FALSE);
+            g_array_free (c_old, TRUE);
           gtk_widget_set_size_request (cat_display, 0, -1);
           gtk_widget_hide (cat_display);
         }
@@ -960,11 +961,20 @@ schedule_search (GObject *obj)
 void
 update_display (void)
 {
+  GtkTreeIter iter;
+  GtkTreePath *path = NULL;
   GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
+
+  gtk_tree_selection_get_selected (sel, NULL, &iter);
+  path = gtk_tree_model_get_path (gtk_tree_view_get_model(GTK_TREE_VIEW(list_view)), &iter);
   update_categories ();
-  if (sel)
-    selection_made(sel, G_OBJECT(gtk_widget_get_toplevel(search_entry)));
   do_search (G_OBJECT (search_entry), search_entry);
+  if (path)
+    {
+      gtk_tree_selection_select_path (sel, path);
+      gtk_tree_path_free (path);
+    }
+  selection_made(sel, G_OBJECT(gtk_widget_get_toplevel(search_entry)));
 }
 
 static gboolean
@@ -1277,11 +1287,11 @@ on_main_focus(GtkWindow *window,  GdkEventExpose *event,gpointer user_data)
 }
 
 static int 
-import_one_file(const gchar *filename)
+import_one_file(const gchar *filename, GError **error)
 {
   int result = 0;
 
-  result = import_vcard(filename);
+  result = import_vcard(filename, error);
 
   return result;
 
@@ -1297,6 +1307,7 @@ on_import_vcard (GtkWidget *widget, gpointer data)
   
   if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK)
     {
+      GError *error = NULL;
       gchar *errstr = NULL;
       int ec = 0, i = 0;
       gchar **files = 
@@ -1304,23 +1315,24 @@ on_import_vcard (GtkWidget *widget, gpointer data)
       gtk_widget_hide(filesel); 
       while (files[i])
         {
-          if (import_one_file(files[i]) < 0) 
+          if (import_one_file(files[i], &error) < 0) 
             {
               gchar *tmp;
               if (!errstr) 
                 errstr=g_strdup("");
               ec++;
-			  tmp = g_strdup_printf("%s\n%s",errstr,strrchr(files[i],'/')+1);
+	      tmp = g_strdup_printf("%s\n%s: %s",errstr,strrchr(files[i],'/')+1,error->message);
               if (errstr) 
                  g_free(errstr);
               errstr = tmp;
+	      g_clear_error(&error);
             }
           i++;  
         }
       if (ec)
         feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
           GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
-          "%s %i %s\n%s",_("Import of"),ec,_("files failed:"),errstr);
+          _("Import of %i files failed:\n%s"), ec, errstr);
       else
         feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
           GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
@@ -1333,6 +1345,48 @@ on_import_vcard (GtkWidget *widget, gpointer data)
   update_display();
 }
 
+static void
+on_export_vcard (GtkWidget *widget, gpointer data)
+{
+  GtkWidget *filesel, *feedbackdlg;
+  gint i = 0, ec = 0;
+  GtkTreeIter iter;
+    
+  filesel = gtk_file_selection_new(_("Export to file..."));
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), "GPE.vcf");
+  gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(filesel), FALSE);
+  
+  if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_OK)
+    {
+      guint uid;
+      const gchar *filename = 
+        gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
+      gtk_widget_hide(filesel);
+      
+      gtk_tree_model_get_iter_first (GTK_TREE_MODEL(list_store), &iter);
+      do 
+        {
+          gtk_tree_model_get (GTK_TREE_MODEL (list_store), &iter, 1, &uid, -1);
+          if (save_to_file (uid, filename, i ? TRUE : FALSE))
+             ec++;
+          i++;
+        }
+      while (gtk_tree_model_iter_next (GTK_TREE_MODEL(list_store), &iter));
+        
+      if (ec)
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
+          GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+          _("Export of %i files OK.  Failed: %i"), i - ec, ec);
+      else
+        feedbackdlg = gtk_message_dialog_new(GTK_WINDOW(mainw),
+          GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+          _("Export successful"));
+      gtk_dialog_run(GTK_DIALOG(feedbackdlg));
+      gtk_widget_destroy(feedbackdlg);
+    }
+  gtk_widget_destroy(filesel);
+}
+
 static GtkWidget *
 create_main (gboolean edit_structure)
 {
@@ -1463,6 +1517,15 @@ create_main (gboolean edit_structure)
   g_signal_connect(G_OBJECT(b), "clicked", G_CALLBACK(on_import_vcard), NULL);
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), b, -1);
 
+  /* export button */
+  pw = gtk_image_new_from_stock (GTK_STOCK_SAVE,
+                                 gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
+  b = gtk_tool_button_new(pw, _("Export"));
+  gtk_tool_item_set_tooltip(b, tooltips, 
+                            _("Export currently listed contacts to a file."), NULL);
+  g_signal_connect(G_OBJECT(b), "clicked", G_CALLBACK(on_export_vcard), NULL);
+  gtk_toolbar_insert(GTK_TOOLBAR(toolbar), b, -1);
+	
   /* another separator */
   b = gtk_separator_tool_item_new();
   gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(b), FALSE);
@@ -1685,23 +1748,20 @@ main (int argc, char *argv[])
   if (ifile)
     {
       int ret;
-      GtkWidget *dialog = NULL;
+      GError *error = NULL;
       
-      ret =  import_one_file(ifile);
-      if (ret)
-       dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
-	                                    GTK_BUTTONS_OK,
-	                                    _("Could not import file %s."),
-	                                     ifile);
-	  else
-        dialog = gtk_message_dialog_new(NULL,0,GTK_MESSAGE_INFO,
-	                                      GTK_BUTTONS_OK,
-	                                      _("File %s imported sucessfully."),
-	                                      ifile);
-      gtk_dialog_run(GTK_DIALOG(dialog));
-      gtk_widget_destroy(dialog);
-      
-      exit (EXIT_SUCCESS);
+      ret =  import_one_file(ifile, &error);
+      if (ret) 
+	{
+	  g_print(_("Could not import file %s: %s.\n"), ifile, error->message);
+	  g_error_free(error);
+	  exit (EXIT_FAILURE);
+	}
+      else
+	{
+	  g_print(_("File %s imported sucessfully.\n"), ifile);      
+	  exit (EXIT_SUCCESS);
+	}
     }
   
   export_init ();
@@ -1724,7 +1784,7 @@ main (int argc, char *argv[])
         p = contacts_new_person();
       edit_person (p, _("My Card"), TRUE);
       gtk_main();
-      if (save_to_file(1, MY_VCARD))
+      if (save_to_file(1, MY_VCARD, FALSE))
         gpe_perror_box(_("Saving vcard failed"));
       exit (EXIT_SUCCESS);
     }
